// chatgpt.js

async function getErrorFix(apiKey, engineCode, motorBrand, model) {
    const prompt = `I am a diesel mechanic trying to diagnose an issue with a ${motorBrand} engine displaying error code ${engineCode}. What are the possible causes and solutions for this error code?`;

    try {
        const response = await fetch("https://api.openai.com/v1/chat/completions", {
            method: "POST",
            headers: {
                "Content-Type": "application/json",
                "Authorization": `Bearer ${apiKey}`
            },
            body: JSON.stringify({
                model: model,
                messages: [
                    { role: "user", content: prompt }
                ],
                temperature: 0.7,
            }),
        });

        if (!response.ok) {
            const errorData = await response.json();
            throw new Error(`API error: ${response.status} ${response.statusText} - ${errorData.error}`);
        }

        const data = await response.json();

        if (data.choices && data.choices.length > 0) {
            return data.choices[0].message.content.trim();
        } else {
            console.error("No choices returned from the API.");
            return "An error occurred. Please try again.";
        }
    } catch (error) {
        console.error(error);
        return "An error occurred. Please check the console for more information.";
    }
}

async function fetchModelsAndPopulateSelect() {
    const apiKey = document.getElementById("api-key").value;
    const errorMessageElement = document.getElementById("error-message");

    // Clear the error message
    errorMessageElement.innerText = "";
    errorMessageElement.classList.add("hidden");

    // Check if the API key is empty
    if (!apiKey) {
        errorMessageElement.innerText = "Please provide an OpenAI API key.";
        errorMessageElement.classList.remove("hidden");
        return false;
    }

    try {
        const response = await fetch("https://api.openai.com/v1/models", {
            method: "GET",
            headers: {
                "Authorization": `Bearer ${apiKey}`
            },
        });

        if (!response.ok) {
            const errorData = await response.json();
            throw new Error(`API error: ${response.status} ${response.statusText} - ${errorData.error}`);
        }

        const data = await response.json();
        const selectElement = document.getElementById("model");

        // Filter the main models
        const mainModels = data.data.filter(model => {
            return model.id.includes("gpt");
        });

        // Remove models ending with numbers
        const filteredModels = mainModels.filter(model => {
            return !/\d$/.test(model.id);
        });

        // Sort models alphabetically
        filteredModels.sort((a, b) => a.id.localeCompare(b.id));

        // Move the GPT model to the beginning of the array
        const gptIndex = filteredModels.findIndex(model => model.id.includes("gpt"));
        if (gptIndex !== -1) {
            const gptModel = filteredModels.splice(gptIndex, 1)[0];
            filteredModels.unshift(gptModel);
        }

        filteredModels.forEach(model => {
            const optionElement = document.createElement("option");
            optionElement.value = model.id;
            optionElement.textContent = model.id;
            selectElement.appendChild(optionElement);
        });

        return true;

    } catch (error) {
        console.error(error);
        errorMessageElement.innerText = "An error occurred. Please check the console for more information.";
        errorMessageElement.classList.remove("hidden");
        return false;
    }
}

// Add this function to handle form visibility
function toggleFormVisibility(showForm) {
    if (showForm) {
        document.getElementById("error-code-form").classList.remove("hidden");
        document.getElementById("reload").classList.add("hidden");
    } else {
        document.getElementById("error-code-form").classList.add("hidden");
    }
}

document.getElementById("fetch-models-button").addEventListener("click", async function () {
    const apiKey = document.getElementById("api-key").value;
    const modelsFetchSuccessful = await fetchModelsAndPopulateSelect(apiKey);

    if (modelsFetchSuccessful) {
        document.getElementById("fetch-models-button").classList.add("hidden");
        document.getElementById("submit").classList.remove("hidden");
    }
});

// Modify the event listener for the error-code-form submit event
document.getElementById("error-code-form").addEventListener("submit", async function (event) {
    event.preventDefault();

    const engineCode = document.getElementById("engine-code").value;
    const motorBrand = document.getElementById("motor-brand").value;
    const apiKey = document.getElementById("api-key").value;
    const model = document.getElementById("model").value;

    // Hide the form and display the loading element
    toggleFormVisibility(false);
    document.getElementById("loading").classList.remove("hidden");

    const result = await getErrorFix(apiKey, engineCode, motorBrand, model);

    // Display the result and the reload button, hide the loading element
    document.getElementById("result").innerText = result;
    document.getElementById("loading").classList.add("hidden");
    document.getElementById("reload").classList.remove("hidden");
});

// Add a new event listener for the reload button
document.getElementById("reload").addEventListener("click", function () {
    // Clear the result and show the form
    document.getElementById("result").innerText = "";
    toggleFormVisibility(true);
});